@@ -13,16 +13,22 @@ unsigned char *buf;
 	int total_read=0;
 
 	if (fd >= 0) {
+		--record_size;
 		while(total_read < record_size && num_read > 0) {
 			num_read = read(fd, buf+total_read, record_size - total_read);
 			total_read = total_read + num_read;
 		}
+		if ((0 == num_read) && (0 < total_read)) {
+                   /* Add newline terminator */
+                   buf [total_read] = '\n';
+                   ++total_read;
+                }
 	}
 #if	AGREP_POINTER
 	else return 0;	/* should not call this function if buf is a pointer to a user-specified region! */
 #else	/*AGREP_POINTER*/
 	else {	/* simulate a file */
-		total_read = (record_size > (agrep_inlen - agrep_inpointer)) ? (agrep_len - agrep_inpointer) : record_size;
+		total_read = (record_size > (agrep_inlen - agrep_inpointer)) ? (agrep_inlen - agrep_inpointer) : record_size;
 		memcpy(buf, agrep_inbuffer + agrep_inpointer, total_read);
 		agrep_inpointer += total_read;
 	}
