@@ -1,6 +1,7 @@
 /* Copyright (c) 1994 Burra Gopal, Udi Manber.  All Rights Reserved. */
 #include "glimpse.h"
 #include "defs.h"
+#include <errno.h>
 
 #if	BG_DEBUG
 extern	FILE	*debug;
@@ -492,7 +493,7 @@ struct offsets **offset_tab;
 char *pattern;
 int patlen;
 int patattr;
-char *index_argv;
+char *index_argv[];
 int index_argc;
 FILE *outfile;
 FILE *partfp;
@@ -520,7 +521,7 @@ int first_time;
 	dest_index_buf[0] = '\n';	/* memagrep needs buffer to begin with '\n' */
 
 	dest_index_set[REAL_PARTITION - 2] = 0;
-	while(fgets(dest_index_buf+1, REAL_INDEX_BUF, f_in)) {
+	while(fgets(dest_index_buf+1, REAL_INDEX_BUF-1, f_in)) {
 #if	BG_DEBUG
 		fprintf(debug, "index-line: %s", dest_index_buf+1);
 #endif	/*BG_DEBUG*/
