@@ -24,6 +24,7 @@
 -----------------------------------------------------------------------*/
 
 #include "glimpse.h"
+#include <errno.h>
 
 #define debugt
 #define BINARY 1
@@ -44,6 +45,9 @@ extern struct stat istbuf;
 extern struct stat excstbuf;
 extern struct stat incstbuf;
 
+void insert_h();
+void insert_index();
+
 extern int ICurrentFileOffset;
 extern int NextICurrentFileOffset;
 
@@ -67,6 +71,7 @@ extern int FilenamesOnStdin;
 extern int UseFilters;
 extern int ByteLevelIndex;
 extern int RecordLevelIndex;
+extern int StoreByteOffset;
 extern int rdelim_len;
 extern char rdelim[MAX_LINE_LEN];
 extern char old_rdelim[MAX_LINE_LEN];
@@ -861,7 +866,7 @@ try_again_1:
 	    }
 	    else if (ret == 2) {
 		/* Some pattern matched but no filter was successful */
-		if (filetype(LIST_GET(name_list, i), 0)) {	/* try to index input file if it satisfies filetype */
+		if (filetype(LIST_GET(name_list, i), 0, NULL, NULL)) {	/* try to index input file if it satisfies filetype */
 		    remove_filename(i, -1);
 		    unlink(outname);
 		    continue;
@@ -901,7 +906,10 @@ try_again_1:
 	    /* buffer always points to a BLOCK_SIZE block of allocated memory */
 	    buffer = buffer_begin;
 	    residue = 0;
-	    if (RecordLevelIndex) NextICurrentFileOffset = ICurrentFileOffset = 1;
+	    if (RecordLevelIndex) {
+		if (!StoreByteOffset) NextICurrentFileOffset = ICurrentFileOffset = 1;
+		else NextICurrentFileOffset = ICurrentFileOffset = 0;
+	    }
 	    for (offset = 0; offset < stbuf.st_size; offset += BLOCK_SIZE) {
 		offset -= residue;
 		if (!RecordLevelIndex) NextICurrentFileOffset = ICurrentFileOffset = offset;
@@ -944,8 +952,9 @@ try_again_1:
 		while ((buffer=(unsigned char *) getword(LIST_GET(name_list, i), word, buffer, buffer_end, &attribute, &next_record)) < buffer_end) {
 		    if (RecordLevelIndex) {
 			if (buffer >= next_record) {
-			    ICurrentFileOffset ++;
 			    next_record = forward_delimiter(buffer, buffer_end, rdelim, rdelim_len, 0);
+			    if (StoreByteOffset) ICurrentFileOffset += next_record - buffer;
+			    else ICurrentFileOffset ++;
 			}
 		    }
 		    /* printf("%s\n", word); */
@@ -974,8 +983,8 @@ try_again_1:
 		}
 		if (RecordLevelIndex) {
 			if (buffer >= next_record) {
-			    ICurrentFileOffset ++;
 			    /* next_record = forward_delimiter(buffer, buffer_end, rdelim, rdelim_len, 0); */
+			    ICurrentFileOffset ++;
 			}
 		}
 		buffer = buffer_begin;
@@ -1067,7 +1076,7 @@ try_again_1:
 	    }
 	    else if (ret == 2) {
 		/* Some pattern matched but no filter was successful */
-		if (filetype(LIST_GET(name_list, i), 0)) {	/* try to index input file if it satisfies filetype */
+		if (filetype(LIST_GET(name_list, i), 0, NULL, NULL)) {	/* try to index input file if it satisfies filetype */
 		    remove_filename(i, -1);
 		    unlink(outname);
 		    continue;
@@ -1107,7 +1116,10 @@ try_again_1:
 	    /* buffer always points to a BLOCK_SIZE block of allocated memory */
 	    buffer = buffer_begin;
 	    residue = 0;
-	    if (RecordLevelIndex) NextICurrentFileOffset = ICurrentFileOffset = 1;
+	    if (RecordLevelIndex) {
+		if (!StoreByteOffset) NextICurrentFileOffset = ICurrentFileOffset = 1;
+		else NextICurrentFileOffset = ICurrentFileOffset = 0;
+	    }
 	    for (offset = 0; offset < stbuf.st_size; offset += BLOCK_SIZE) {
 		offset -= residue;
 		if (!RecordLevelIndex) NextICurrentFileOffset = ICurrentFileOffset = offset;
@@ -1150,8 +1162,9 @@ try_again_1:
 		while ((buffer=(unsigned char *) getword(LIST_GET(name_list, i), word, buffer, buffer_end, &attribute, &next_record)) < buffer_end) {
 		    if (RecordLevelIndex) {
 			if (buffer >= next_record) {
-			    ICurrentFileOffset ++;
 			    next_record = forward_delimiter(buffer, buffer_end, rdelim, rdelim_len, 0);
+			    if (StoreByteOffset) ICurrentFileOffset += next_record - buffer;
+			    else ICurrentFileOffset ++;
 			}
 		    }
 		    /* printf("%s\n", word); */
@@ -1180,8 +1193,8 @@ try_again_1:
 		}
 		if (RecordLevelIndex) {
 			if (buffer >= next_record) {
-			    ICurrentFileOffset ++;
 			    /* next_record = forward_delimiter(buffer, buffer_end, rdelim, rdelim_len, 0); */
+			    ICurrentFileOffset ++;
 			}
 		}
 		buffer = buffer_begin;
@@ -1236,6 +1249,7 @@ function: insert the word to appropriate position in the table.
           otherwise create a new token.
 THERE ARE NO STATE CHANGES UNLESS WE ARE SURE THAT MALLOCS WON'T FAIL: BG
 ---------------------------------------------------------------------------*/
+void
 insert_h(word, pn, attribute)
 char *word;
 int  pn;
@@ -1273,7 +1287,8 @@ int attribute;
 	traverse1();
 	init_hash_table();
 	tried_once = 1;	/* memory allocation failed in malloc#1 */
-	return insert_h(word, pn, attribute);	/* next call can't fail here since traverse() calls *allfree() */
+	insert_h(word, pn, attribute);	/* next call can't fail here since traverse() calls *allfree() */
+        return;
     }
 
     if((tp->word = (char *) wordalloc(sizeof(char) * (wordlen+1))) == NULL) {
@@ -1288,7 +1303,8 @@ int attribute;
 	traverse1();
 	init_hash_table();
 	tried_once = 2;	/* memory allocation failed in malloc#2 */
-	return insert_h(word, pn, attribute);	/* next call can't fail here or above since traverse() calls *allfree() */
+	insert_h(word, pn, attribute);	/* next call can't fail here or above since traverse() calls *allfree() */
+        return;
     }
     strcpy(tp->word, word);
     tp->attribute = attribute;
@@ -1308,7 +1324,8 @@ int attribute;
 	traverse1();
 	init_hash_table();
 	tried_once = 3;	/* memory allocation failed in malloc#3 */
-	return insert_h(word, pn, attribute);	/* next call can't fail here or above or above-above since traverse() calls *allfree() */
+	insert_h(word, pn, attribute);	/* next call can't fail here or above or above-above since traverse() calls *allfree() */
+        return;
     }
     icount++;
 
@@ -1354,6 +1371,7 @@ tp points to the token structure. so, tp->ip is always the active
 indices structure.
 THERE ARE NO STATE CHANGES UNLESS WE ARE SURE THAT MALLOCS WON'T FAIL: BG
 ------------------------------------------------------------------- */
+void
 insert_index(tp, pn)
 struct token *tp;               /* insert a index into a indices structure */
 int pn;
@@ -1402,7 +1420,8 @@ int pn;
 	traverse1();
 	init_hash_table();
 	tried_once = 1;	/* memory allocation failed in malloc#1 */
-	return insert_index(tp, pn);
+	insert_index(tp, pn);
+        return;
     }
     icount++;
 
@@ -1476,6 +1495,9 @@ FILE *f1, *f2, *f3;
     unsigned int attr1, attr2;
     int x=0, y=0, even_words = 1;
 
+    int merge_buf_size=REAL_INDEX_BUF;
+
+
     /* LOOK OUT FOR: [memset, fgets, endpt-forloop, scanword] 4-tuples: invariant */
 
 #if	debug
@@ -1636,7 +1658,16 @@ printf("in merge_in()\n"); fflush(stdout);
 		    }
 
 		    even_words = 1;
+		    /* prevent buffer overflows */	
 		    if (merge_len > 0) {
+
+			if (merge_len > merge_buf_size - 6) {
+				merge_buf_size += 20;
+				merge_index_buf = realloc(merge_index_buf, merge_buf_size);
+fprintf(stderr,"Realloc + 20 new size is %d",merge_buf_size);
+				memset(merge_index_buf + merge_len,'\0',merge_buf_size-merge_len);
+			}
+
 			if(OneFilePerBlock &&
 			   ((src_mark == ALL_INDEX_MARK) ||
 			    (dest_mark == ALL_INDEX_MARK) ||
@@ -1671,7 +1702,13 @@ printf("in merge_in()\n"); fflush(stdout);
 			}
 			if (OneFilePerBlock) {
 			    for (i=0; i<round(file_num, 8*sizeof(int)); i++) dest_index_set[i] |= src_index_set[i];	/* take union */
-			    for (i=0; i<round(file_num, 8*sizeof(int)); i++)
+			    for (i=0; i<round(file_num, 8*sizeof(int)); i++) {
+				if (merge_len > (merge_buf_size - 3*8*sizeof(int) - 1)) {
+					merge_buf_size *= 2;
+					merge_index_buf = realloc(merge_index_buf, merge_buf_size);
+fprintf(stderr,"Had to Realloc merge buffer (#2), new size is %d\n",merge_buf_size);
+					memset(merge_index_buf + merge_len,'\0',merge_buf_size-merge_len);
+				}
 				if (dest_index_set[i])
 				    for (j=0; j<8*sizeof(int); j++)
 					if (dest_index_set[i] & mask_int[j]) {
@@ -1705,18 +1742,36 @@ printf("in merge_in()\n"); fflush(stdout);
 							merge_index_buf[merge_len ++] = x&0x000000ff;
 						}
 					}
+			    }
 			    if (!even_words && (file_num > MaxNum8bPartition) && (file_num <= MaxNum12bPartition))
 				merge_index_buf[merge_len ++] = y;
 			}
 			else {	/* normal indexing */
 			    for (i=0; i<src_num; i++) {
 				merge_index_buf[merge_len++] = src_index_set[i];
+				 /* Prevent buffer overflow */
+				if (merge_len > (merge_buf_size - 3)) {
+					merge_buf_size *= 2;
+					merge_index_buf = realloc(merge_index_buf, merge_buf_size);
+fprintf(stderr,"Had to Realloc merge buffer (#3), new size is %d\n",merge_index_buf);
+					memset(merge_index_buf + merge_len,'\0',merge_buf_size-merge_len);
+				}
 			    }
+
 			    for (j=0; j<dest_num; j++) {
 				for (i=0; i<src_num; i++)
 				    if (dest_index_set[j] == src_index_set[i]) break;
 				if (i>=src_num)     /* did not find match */
 				    merge_index_buf[merge_len++] = dest_index_set[j];
+
+				 /* Prevent buffer overflow */
+				if (merge_len > (merge_buf_size - 2)) {
+					merge_buf_size *= 2;
+					merge_index_buf = realloc(merge_index_buf, merge_buf_size);
+fprintf(stderr,"Realloc #4, new size is %d\n",merge_index_buf);
+					memset(merge_index_buf + merge_len,'\0',merge_buf_size-merge_len);
+				}
+				
 				/* Doesn't matter if dest_index_set is int-array (merge_index_buf being char array) since dest_index_set has only a char */
 			    }
 			}
@@ -1748,7 +1803,7 @@ printf("in merge_in()\n"); fflush(stdout);
 
                     /* append the indices of word1 to the buffer */
                     if (dest_index_buf[bdx2] > 0) {
-                        while(dest_index_buf[bdx2]>0) merge_index_buf[bdx++] = dest_index_buf[bdx2++];  /* '\n' gets copied */
+                        while((dest_index_buf[bdx2]>0)&&(bdx2<REAL_INDEX_BUF)) merge_index_buf[bdx++] = dest_index_buf[bdx2++];  /* '\n' gets copied */
                         merge_index_buf[bdx] = '\0';
                     }
                     /* else, no need to copy anything */
@@ -1970,7 +2025,8 @@ purge_index()
 	fscanf(i_in, "%%%d\n", &onefileperblock);
 	if (ByteLevelIndex) fprintf(i_out, "%%-%d\n", new_file_num);        /* #of files might have changed due to -f/-a */
 	else fprintf(i_out, "%%%d\n", new_file_num);        /* This was the stupidest thing of all! */
-	fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
+	if ( !fscanf(i_in, "%%%d\n", &structuredindex) )  /* New file format may not have delimiter.  Fixed 10/25/99 as per mhubin. --GV */
+             fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
 	if (structuredindex <= 0) structuredindex = 0;
 	if (RecordLevelIndex) fprintf(i_out, "%%-2 %s\n", old_rdelim);     /* robint@zedcor.com */
 	else fprintf(i_out, "%%%d\n", attr_num);     /* attributes might have been added during last merge */
