@@ -224,7 +224,7 @@ extract_info(name, buffer, num_read, suffix_index, xinfo, max_len)
 	   crashes. We need an extra byte for the terminating 0, so loop
 	   only up to max_len - 1.
 	   - CV 01/10/00 */
-	while ((i<num_read) && (k<max_len - 1)) {
+	while ((i<num_read) && (k<max_len  - strlen(name) - 3)) {
 		if (buffer[i] != '<') {
 			if ((buffer[i] == '\0') || (buffer[i] == '\n')) {
 				xinfo[k++] = ' ';	/* must convert whole title to one line */
