@@ -19,12 +19,13 @@ extern FILE  *MESSAGEFILE;	/* file descriptor for important messages meant for t
 extern char  INDEX_DIR[MAX_LINE_LEN];
 extern struct stat istbuf;
 
-extern char	*TEMP_DIR; /* directory to store glimpse temporary files, usually /tmp unless -T is specified */
 #ifdef BUILDCAST
 /* TEMP_DIR is normally defined in ../main.c; if we're building
  * buildcast, that's not linked in, so we need to define one here. */
 /* char * TEMP_DIR = NULL; */
 static char * TEMP_DIR = "/tmp";
+#else
+extern char	*TEMP_DIR; /* directory to store glimpse temporary files, usually /tmp unless -T is specified */
 #endif /* BUILDCAST */
 extern int indexable_char[256];
 extern int GenerateHash;
@@ -1077,6 +1078,7 @@ cleanup()
     unlink(s);
     sprintf(s, "%s/.glimpse_apply.%d", INDEX_DIR, getpid());
     unlink(s);
+    return(1); /* We return 1, because function expect int as return type */
 }
 
 #if	!BUILDCAST
@@ -1136,5 +1138,6 @@ int	flag;
 	fprintf(stderr, "\n");
 	fprintf(stderr, "For questions about glimpse, please contact: `%s'\n", GLIMPSE_EMAIL);
 	exit(1);
+	return(1); /* We type return 1, because function expect int as return type */
 }
 #endif	/*!BUILDCAST*/
