@@ -266,7 +266,7 @@ static void load_dyn_filters(void)
 	len = strlen(filter_command[i]);
 	if (len > 4) {
 	    /* find location in string where .so suffix should be */
-	    so_pos = filter_command[i] + len - 3;
+	    so_pos = (char *)(filter_command[i] + len - 3);
 	    if (strcmp(so_pos, ".so") == 0) {
 		/* fprintf(stderr, "Loading %s\n", filter_command[i]); */
 		if ((filter_handle[i] = dlopen(filter_command[i],
