@@ -3,6 +3,7 @@
 #include "glimpse.h"
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <errno.h>
 
 extern int BigFilenameHashTable;
 extern int DeleteFromIndex;
@@ -681,7 +682,8 @@ save_data_structures()
 	    fputs(indexnumberbuf, i_out);
 	    fscanf(i_in, "%%%d\n", &onefileperblock);
 	    fprintf(i_out, "%%%d\n", onefileperblock);	/* If #of files change, then they are added to a new partition, which is updated above */
-	    fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
+	    if ( !fscanf(i_in, "%%%d\n", &structuredindex) ) /* temp_rdelim may not be present in new-style indexes.  Fixed by mhubin 10/25/99 --GV */
+                 fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
 	    if (structuredindex <= 0) structuredindex = 0;
 	    if (RecordLevelIndex) fprintf(i_out, "%%-2 %s\n", old_rdelim);	/* robint@zedcor.com (CANNOT HAPPEN SINCE RecordLevel AND Strucured ARE NOT COMPATIBLE!!!) */
 	    else fprintf(i_out, "%%%d\n", attr_num);	/* attributes might have been added during last merge */
@@ -742,8 +744,9 @@ save_data_structures()
         fscanf(i_in, "%%%d\n", &onefileperblock);
         if (ByteLevelIndex) fprintf(i_out, "%%-%d\n", file_num);	/* #of files might have changed due to -f/-a */
 	else fprintf(i_out, "%%%d\n", file_num);	/* This was the stupidest thing of all! */
-        fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
-        if (structuredindex <= 0) structuredindex = 0;
+        if ( !fscanf(i_in, "%%%d\n", &structuredindex) ) /* 10/25/99 as per mhubin --GV */
+               fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
+	if (structuredindex <= 0) structuredindex = 0;
 	if (RecordLevelIndex) fprintf(i_out, "%%-2 %s\n", old_rdelim);	/* robint@zedcor.com */
         else fprintf(i_out, "%%%d\n", attr_num);	/* attributes might have been added during last merge */
 
@@ -949,6 +952,8 @@ merge_splits()
 	getchar();
 #endif	/*0*/
 
+	temp_rdelim[0] = '\0';  /* Initialize in case not read. 10/25/99 --GV */
+
 	sprintf(s, "%s/%s", INDEX_DIR, P_TABLE);
 	if ((p_in = fopen(s, "r")) == NULL) {
 		fprintf(stderr, "cannot open for reading: %s\n", s);
@@ -972,7 +977,8 @@ merge_splits()
 	fputs(indexnumberbuf, i_out);
 	fscanf(i_in, "%%%d\n", &onefileperblock);
 	fprintf(i_out, "%%%d\n", onefileperblock);
-	fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
+      	if ( !fscanf(i_in, "%%%d\n", &structuredindex) )    /* 10/25/99 as per mhubin --GV */
+             fscanf(i_in, "%%%d%s\n", &structuredindex, temp_rdelim);
 	if (structuredindex == -2) recordlevelindex = 1;
 	if (structuredindex <= 0) structuredindex = 0;
 	if (recordlevelindex) fprintf(i_out, "%%-2 %s\n", temp_rdelim);
