@@ -73,7 +73,9 @@ CONT:
 	if(buffer[i] == 'M') {
 		if((j=memlook("\nM", &buffer[i], length-i)) < 80) return(1);
 	}
-	else return(0);
+	/* pab23feb98: return(0) formerly in else of buffer[i]=='M' compare.
+	 * I.e., fellthrough with invalid return if nested if failed. */
+	return(0);
 }
 
 int
