@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: template.c,v 1.2 2003/11/13 05:17:35 golda Exp $";
+static char rcsid[] = "$Id: template.c,v 1.3 2006/03/25 02:13:55 root Exp $";
 /*
  *  template.c - SOIF Object ("template") processing code for Harvest 
  *
@@ -12,42 +12,6 @@ static char rcsid[] = "$Id: template.c,v 1.2 2003/11/13 05:17:35 golda Exp $";
  *          Darren R. Hardy of the University of Colorado at Boulder.
  *          Udi Manber of the University of Arizona.
  *          Michael F. Schwartz of the University of Colorado at Boulder. 
- *  
- *  This copyright notice applies to all code in Harvest other than
- *  subsystems developed elsewhere, which contain other copyright notices
- *  in their source text.
- *  
- *  The Harvest software was developed by the Internet Research Task
- *  Force Research Group on Resource Discovery (IRTF-RD).  The Harvest
- *  software may be used for academic, research, government, and internal
- *  business purposes without charge.  If you wish to sell or distribute
- *  the Harvest software to commercial clients or partners, you must
- *  license the software.  See
- *  http://harvest.cs.colorado.edu/harvest/copyright,licensing.html#licensing.
- *  
- *  The Harvest software is provided ``as is'', without express or
- *  implied warranty, and with no support nor obligation to assist in its
- *  use, correction, modification or enhancement.  We assume no liability
- *  with respect to the infringement of copyrights, trade secrets, or any
- *  patents, and are not responsible for consequential damages.  Proper
- *  use of the Harvest software is entirely the responsibility of the user.
- *  
- *  For those who are using Harvest for non-commercial purposes, you may
- *  make derivative works, subject to the following constraints:
- *  
- *  - You must include the above copyright notice and these accompanying 
- *    paragraphs in all forms of derivative works, and any documentation 
- *    and other materials related to such distribution and use acknowledge 
- *    that the software was developed at the above institutions.
- *  
- *  - You must notify IRTF-RD regarding your distribution of the 
- *    derivative work.
- *  
- *  - You must clearly notify users that your are distributing a modified 
- *    version and not the original Harvest software.
- *  
- *  - Any derivative product is also subject to the restrictions of the 
- *    copyright, including distribution and use limitations.
  */
 #include <stdio.h>
 #include <string.h>
@@ -75,7 +39,7 @@ int vsize;
 	l->data = xmalloc(sizeof(AVPair));
 	l->data->value = xmalloc(vsize + 1);
 
-	l->data->attribute = strdup(attr);
+	l->data->attribute = (char *)strdup(attr);
 	l->data->vsize = vsize;
 	memcpy(l->data->value, value, l->data->vsize);
 	l->data->value[l->data->vsize] = '\0';
@@ -287,10 +251,10 @@ char *url;
 
 	t = xmalloc(sizeof(Template));
 	if (type == NULL)
-		t->template_type = strdup("FILE");
+		t->template_type = (char *)strdup("FILE");
 	else
-		t->template_type = strdup(type);
-	t->url = strdup(url);
+		t->template_type = (char *)strdup(type);
+	t->url = (char *)strdup(url);
 	t->list = NULL;
 	t->offset = -1;
 	t->length = -1;
@@ -583,12 +547,12 @@ Template *parse_template()
 	template->offset = inputoffset;
 	/* Get Template-Type */
 	grab_token();
-	template->template_type = strdup(buf);
+	template->template_type = (char *)strdup(buf);
 
 	/* Get URL */
 	skip_whitespace_and('{');
 	grab_token();
-	template->url = strdup(buf);
+	template->url = (char *)strdup(buf);
 	template->list = NULL;
 
 #ifdef DEBUG
@@ -602,7 +566,7 @@ Template *parse_template()
 		if (done == 1)
 			break;
 		grab_attribute();
-		attribute = strdup(buf);
+		attribute = (char *)strdup(buf);
 #ifdef DEBUG
 		log("Grabbed Attribute: %s\n", attribute);
 #endif
